package org.jboss.security.ssl;

package .var.folders.cl.9twcgjfx4vsg5zl73c6wxmvm0000gn.T;

import java.io.IOException;
import java.net.InetAddress;
import java.net.Socket;
import java.net.UnknownHostException;
import java.util.Arrays;
import java.util.Hashtable;
import javax.naming.InitialContext;
import javax.net.SocketFactory;
import javax.net.ssl.HandshakeCompletedEvent;
import javax.net.ssl.HandshakeCompletedListener;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSession;
import javax.net.ssl.SSLSocket;
import javax.net.ssl.SSLSocketFactory;
import org.jboss.logging.Logger;
import org.jboss.security.SecurityDomain;
import org.jboss.security.ssl.ClientSocketFactory;
import org.jboss.security.ssl.Context;

public class DomainSocketFactory extends SSLSocketFactory implements HandshakeCompletedListener {
  public static final String HANDSHAKE_COMPLETE_LISTENER = "org.jboss.security.ssl.HandshakeCompletedListener";
  
  private static Logger log = Logger.getLogger(org.jboss.security.ssl.DomainSocketFactory.class);
  
  private transient SecurityDomain securityDomain;
  
  private transient SSLContext sslCtx = null;
  
  private boolean wantsClientAuth = true;
  
  private boolean needsClientAuth = false;
  
  public DomainSocketFactory(SecurityDomain securityDomain) throws IOException {
    if (securityDomain == null)
      throw new IOException("The securityDomain may not be null"); 
    this.securityDomain = securityDomain;
  }
  
  public SecurityDomain getSecurityDomain() {
    return this.securityDomain;
  }
  
  public void setSecurityDomain(SecurityDomain securityDomain) {
    this.securityDomain = securityDomain;
  }
  
  public boolean isWantsClientAuth() {
    return this.wantsClientAuth;
  }
  
  public void setWantsClientAuth(boolean wantsClientAuth) {
    this.wantsClientAuth = wantsClientAuth;
  }
  
  public boolean isNeedsClientAuth() {
    return this.needsClientAuth;
  }
  
  public void setNeedsClientAuth(boolean needsClientAuth) {
    this.needsClientAuth = needsClientAuth;
  }
  
  public Socket createSocket(String serverHost, int serverPort) throws IOException, UnknownHostException {
    InetAddress serverAddr = InetAddress.getByName(serverHost);
    return createSocket(serverAddr, serverPort);
  }
  
  public Socket createSocket(String serverHost, int serverPort, InetAddress clientAddr, int clientPort) throws IOException, UnknownHostException {
    InetAddress serverAddr = InetAddress.getByName(serverHost);
    return createSocket(serverAddr, serverPort, clientAddr, clientPort);
  }
  
  public Socket createSocket(InetAddress serverAddr, int serverPort) throws IOException {
    return createSocket(serverAddr, serverPort, null, 0);
  }
  
  public Socket createSocket(InetAddress serverAddr, int serverPort, InetAddress clientAddr, int clientPort) throws IOException {
    initSSLContext();
    SSLSocketFactory factory = this.sslCtx.getSocketFactory();
    SSLSocket socket = (SSLSocket)factory.createSocket(serverAddr, serverPort, clientAddr, clientPort);
    String[] supportedProtocols = socket.getSupportedProtocols();
    log.debug("Supported protocols: " + Arrays.<String>asList(supportedProtocols));
    String[] protocols = supportedProtocols;
    socket.setEnabledProtocols(protocols);
    socket.addHandshakeCompletedListener(this);
    socket.setNeedClientAuth(this.needsClientAuth);
    socket.setWantClientAuth(this.wantsClientAuth);
    return socket;
  }
  
  public Socket createSocket(Socket s, String host, int port, boolean autoClose) throws IOException {
    initSSLContext();
    SSLSocketFactory factory = this.sslCtx.getSocketFactory();
    SSLSocket socket = (SSLSocket)factory.createSocket(s, host, port, autoClose);
    socket.addHandshakeCompletedListener(this);
    return socket;
  }
  
  public String[] getDefaultCipherSuites() {
    String[] cipherSuites = new String[0];
    try {
      initSSLContext();
      SSLSocketFactory factory = this.sslCtx.getSocketFactory();
      cipherSuites = factory.getDefaultCipherSuites();
    } catch (IOException e) {
      log.error("Failed to get default SSLSocketFactory", e);
    } 
    return cipherSuites;
  }
  
  public String[] getSupportedCipherSuites() {
    String[] cipherSuites = new String[0];
    try {
      initSSLContext();
      SSLSocketFactory factory = this.sslCtx.getSocketFactory();
      cipherSuites = factory.getSupportedCipherSuites();
    } catch (IOException e) {
      log.error("Failed to get default SSLSocketFactory", e);
    } 
    return cipherSuites;
  }
  
  public static SocketFactory getDefault() {
    org.jboss.security.ssl.DomainSocketFactory ssf = null;
    try {
      InitialContext iniCtx = new InitialContext();
      SecurityDomain sd = (SecurityDomain)iniCtx.lookup("java:/jaas/other");
      ssf = new org.jboss.security.ssl.DomainSocketFactory(sd);
    } catch (Exception e) {
      log.error("Failed to create default SocketFactory", e);
    } 
    return ssf;
  }
  
  public void handshakeCompleted(HandshakeCompletedEvent event) {
    Logger log = Logger.getLogger(ClientSocketFactory.class);
    if (log.isTraceEnabled()) {
      String cipher = event.getCipherSuite();
      SSLSession session = event.getSession();
      String peerHost = session.getPeerHost();
      log.debug("SSL handshakeCompleted, cipher=" + cipher + ", peerHost=" + peerHost);
    } 
    try {
      Hashtable env = System.getProperties();
      HandshakeCompletedListener listener = (HandshakeCompletedListener)env.get("org.jboss.security.ssl.HandshakeCompletedListener");
      if (listener != null)
        listener.handshakeCompleted(event); 
    } catch (Throwable e) {
      log.debug("Failed to foward handshakeCompleted", e);
    } 
  }
  
  private void initSSLContext() throws IOException {
    if (this.sslCtx != null)
      return; 
    this.sslCtx = Context.forDomain(this.securityDomain);
  }
  
  public DomainSocketFactory() {}
}
